/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.service;

import org.sonar.commons.configuration.Property;
import org.sonar.commons.database.DatabaseManager;

import javax.persistence.NoResultException;
import javax.persistence.Query;
import java.util.List;
import java.util.Properties;

public class PropertiesDao {

  private DatabaseManager databaseManager;

  public PropertiesDao(DatabaseManager databaseManager) {
    this.databaseManager = databaseManager;
  }

  public Property getProperty(String propertyKey) {
    Query query = databaseManager.createNamedQuery(Property.SQL_SELECT_PROPERTY);
    query.setParameter("property_key", propertyKey);
    try {
      return (Property) query.getSingleResult();
    } catch (NoResultException ex) {
      return null;
    }
  }

  public Properties getProperties() {
    Query query = databaseManager.createQuery("from " + Property.class.getSimpleName());
    Properties result = new Properties();
    List<Property> dbProps = query.getResultList();
    for (Property dbProp : dbProps) {
      result.setProperty(dbProp.getKey(), dbProp.getValue());
    }
    return result;
  }

  public void createProperty(String key, String value) {
    Property property = new Property(key, value);
    databaseManager.save(property);
  }

  public void updateProperty(Property property){
    databaseManager.merge(property);
  }

}
