/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons.configuration;

import org.apache.commons.configuration.AbstractConfiguration;
import org.apache.commons.configuration.event.ConfigurationEvent;
import org.apache.commons.configuration.event.ConfigurationListener;
import org.sonar.commons.database.DatabaseManager;
import org.sonar.commons.database.DatabaseManagerFactory;

import javax.persistence.Query;

public class DatabaseConfigurationListener implements ConfigurationListener {
  private DatabaseManagerFactory databaseManagerFactory = null;

  public DatabaseConfigurationListener(DatabaseManagerFactory dmf) {
    this.databaseManagerFactory = dmf;
  }

  public void configurationChanged(ConfigurationEvent event) {
    if (event.isBeforeUpdate()) {
      if (event.getType() == AbstractConfiguration.EVENT_SET_PROPERTY) {
        save(event.getPropertyName(), event.getPropertyValue());

      } else if (event.getType() == AbstractConfiguration.EVENT_CLEAR_PROPERTY) {
        delete(event.getPropertyName());
      }
    }
  }

  protected void save(String key, Object value) {
    DatabaseManager database = databaseManagerFactory.getDatabaseManager();
    database.start();
    try {
      delete(key, database);

      database.save(new Property(key, value.toString()));
      database.commit();

    } finally {
      database.stop();
    }

  }

  protected void delete(String key) {
    DatabaseManager database = databaseManagerFactory.getDatabaseManager();
    database.start();
    try {
      delete(key, database);
      database.commit();

    } finally {
      database.stop();
    }
  }

  private void delete(String key, DatabaseManager database) {
    Query query = database.createQuery("from Property p where p.key=:key");
    query.setParameter("key", key);
    for (Object obj : query.getResultList()) {
      database.remove(obj);
    }
  }
}
